"{ Package: 'jv:libgdbs/tests' }"

TestCase subclass:#GDBParserTests
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:'GDBCommandStatus'
	category:'GDB-Private-Tests'
!


!GDBParserTests methodsFor:'tests - C strings'!

test_c_string_01

    self assert: (GDBParser on: '"Hello" xxx') parseCString = 'Hello'.
    self assert: (GDBParser on: '"\"Hello\"" xxx') parseCString = '"Hello"'.
    self assert: (GDBParser on: '"\H\e\l\l\o" xxx') parseCString = 'Hello'.
    self assert: (GDBParser on: '"Hel\nlo" xxx') parseCString = 'Hel
lo'.  
    self assert: (GDBParser on: '"X\xE1X" xxx') parseCString = 'XX'.

    self assert: (GDBParser on: '"warning: File \"/home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/stx/.gdbinit\" auto-loading has been declined by your `auto-load safe-path'' set to \"$debugdir:$datadir/auto-load\".\n"')
        parseCString = 'warning: File "/home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/stx/.gdbinit" auto-loading has been declined by your `auto-load safe-path'' set to "$debugdir:$datadir/auto-load".
'.

    "Created: / 28-05-2014 / 00:05:31 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 03-06-2014 / 00:23:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBParserTests methodsFor:'tests - examples'!

test_simple_example_01

    | parser events |

    parser := GDBParser on:
'^done,bkpt={number="1",type="breakpoint",disp="keep",enabled="y",addr="0x08048564",func="main",file="myprog.c",fullname="/home/nickrob/myprog.c",line="68",thread-groups=["i1"],times="0"}
(gdb)
'.
    events := parser parseOutput.

    self assert: events size == 1.
    self assert: events first isCommandResultEvent.
    self assert: events first result status == CommandStatusDone.
    self assert: events first result value key = 'bkpt'.
    self assert: (events first result value value propertyAt: 'addr') = '0x08048564'

    "Created: / 30-05-2014 / 23:53:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 02-06-2014 / 23:04:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_simple_example_02

    | parser events |

    parser := GDBParser on:
'1234^done
(gdb)
'.
    events := parser parseOutput.

    self assert: events size == 1.
    self assert: events first isCommandResultEvent.
    self assert: events first result status == CommandStatusDone.
    self assert: events first token == 1234

    "Created: / 03-06-2014 / 00:50:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

test_simple_session_01

    | parser events |

    parser := GDBParser on:'~"GNU gdb (GDB) 7.5-ubuntu\n"
~"Copyright (C) 2012 Free Software Foundation, Inc.\n"
~"License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\nThis is free software: you are free to change and redistribute it.\nThere is NO WARRANTY, to the extent permitted by law.  Type \"show copying\"\nand \"show warranty\" for details.\n"
~"This GDB was configured as \"x86_64-linux-gnu\".\nFor bug reporting instructions, please see:\n"
~"<http://www.gnu.org/software/gdb/bugs/>...\n"
~"Reading symbols from /home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/jv/libgdbs/tests/c/factorial..."
~"done.\n"
(gdb) 
'.
    events := parser parseOutput.

    self assert: events size == 7.
    self assert: events first isConsoleOutputEvent.
    self assert: events first value = 'GNU gdb (GDB) 7.5-ubuntu
'.
    self assert: events last isConsoleOutputEvent.
    self assert: events last value = 'done.
'.

    parser := GDBParser on:'^done,bkpt={number="1",type="breakpoint",disp="keep",enabled="y",addr="0x0000000000400556",func="main",file="factorial.c",fullname="/home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/jv/libgdbs/tests/c/factorial.c",line="12",times="0",original-location="main"}
(gdb) 
'.
    events := parser parseOutput.

    self assert: events size == 1.
    self assert: events first result status == CommandStatusDone.
    self assert: events first result value key = 'bkpt'.
    self assert: (events first result value value propertyAt: 'addr') = '0x0000000000400556'.


    parser := GDBParser on:'~"Breakpoint 2 at 0x400527: file factorial.c, line 4.\n"
=breakpoint-created,bkpt={number="2",type="breakpoint",disp="keep",enabled="y",addr="0x0000000000400527",func="factorial",file="factorial.c",fullname="/home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/jv/libgdbs/tests/c/factorial.c",line="4",times="0",original-location="factorial"}
^done
(gdb)
'.
    events := parser parseOutput.

    self assert: events size == 3.
    self assert: events first isConsoleOutputEvent.
    self assert: events first value = 'Breakpoint 2 at 0x400527: file factorial.c, line 4.
'.
    self assert: events second isNotificationEvent.
    self assert: events second type = 'breakpoint-created'.

    parser := GDBParser on:'&"info break\n"
~"Num     Type           Disp Enb Address            What\n"
~"1       breakpoint     keep y   0x0000000000400556 in main at factorial.c:12\n"
~"2       breakpoint     keep y   0x0000000000400527 in factorial at factorial.c:4\n"
^done
(gdb)
'.
    events := parser parseOutput.

    self assert: events size == 5.
    self assert: events first isLogOutputEvent.
    self assert: events first value = 'info break
'.
    self assert: events second isConsoleOutputEvent.
    self assert: events third isConsoleOutputEvent.
    self assert: events fourth isConsoleOutputEvent.
    self assert: events fifth isCommandResultEvent.

    parser := GDBParser on:'&"run\n"
~"Starting program: /home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/jv/libgdbs/tests/c/factorial \n"
=thread-group-started,id="i1",pid="17240"
=thread-created,id="1",group-id="i1"
^running
*running,thread-id="all"
(gdb)
'.

    events := parser parseOutput.

    self assert: events size == 6.
    self assert: events third isNotificationEvent.
    self assert: events third type = 'thread-group-started'.
    self assert: events fifth isCommandResultEvent.
    self assert: events fifth result status == CommandStatusDone.

    self assert: events sixth isExecutionEvent.
    self assert: events sixth type = 'running'.


    parser := GDBParser on:'=library-loaded,id="/lib64/ld-linux-x86-64.so.2",target-name="/lib64/ld-linux-x86-64.so.2",host-name="/lib64/ld-linux-x86-64.so.2",symbols-loaded="0",thread-group="i1"
=library-loaded,id="/lib/x86_64-linux-gnu/libc.so.6",target-name="/lib/x86_64-linux-gnu/libc.so.6",host-name="/lib/x86_64-linux-gnu/libc.so.6",symbols-loaded="0",thread-group="i1"
=breakpoint-modified,bkpt={number="1",type="breakpoint",disp="keep",enabled="y",addr="0x0000000000400556",func="main",file="factorial.c",fullname="/home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/jv/libgdbs/tests/c/factorial.c",line="12",times="1",original-location="main"}
~"\nBreakpoint "
~"1, main (argc=1, argv=0x7fffffffe0a8) at factorial.c:12\n"
~"12\t\tint i = 5;\n"
*stopped,reason="breakpoint-hit",disp="keep",bkptno="1",frame={addr="0x0000000000400556",func="main",args=[{name="argc",value="1"},{name="argv",value="0x7fffffffe0a8"}],file="factorial.c",fullname="/home/jv/Private/Projects/SmalltalkX/sources/branches/jv1/build/jv/libgdbs/tests/c/factorial.c",line="12"},thread-id="1",stopped-threads="all",core="1"
(gdb)
'.

    events := parser parseOutput.


    self assert: events size == 7.
    self assert: (events at:7) isExecutionEvent.
    self assert: (events at:7) type = 'stopped'.

    "Created: / 01-06-2014 / 22:57:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 02-06-2014 / 23:05:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBParserTests class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

